#cython: language_level=3

from .capi.obidms       cimport OBIDMS_p
from .capi.obidmscolumn cimport OBIDMS_column_p
from .capi.obiview      cimport Obiview_p
from .capi.obitypes     cimport obiversion_t, OBIType_t, index_t
from ._obitaxo          cimport OBI_Taxonomy

cdef class OBIDMS_column:

    cdef  OBIDMS_column_p*   pointer
    cdef  OBIDMS             dms
    cdef  Obiview_p          view
    cdef  str                data_type
    cdef  str                dms_name
    cdef  str                column_name
    cdef  index_t            nb_elements_per_line
    cdef  list               elements_names
        
    cpdef         update_pointer(self)
    cpdef list    get_elements_names(self)
    cpdef str     get_data_type(self)
    cpdef index_t get_nb_lines_used(self)
    cpdef str     get_creation_date(self)
    cpdef str     get_comments(self)
    cpdef         close(self)
    @staticmethod
    cdef  object  get_subclass_type(OBIDMS_column_p column_p)


cdef class OBIDMS_column_multi_elts(OBIDMS_column):
    
    cpdef set_line(self, index_t line_nb, dict values)


cdef class OBIDMS_column_line:

    cdef OBIDMS_column column
    cdef index_t index


cdef class OBIView:
    
    cdef Obiview_p pointer
    cdef str       name
    cdef str       comments
    cdef dict      columns
    cdef dict      columns_pp    # TODO this dict might be unnecessary
    cdef OBIDMS    dms
    
    cpdef delete_column(self, str column_name)
    cpdef add_column(self, 
                     str column_name, 
                     obiversion_t version_number=*, 
                     str type=*,
                     index_t nb_lines=*, 
                     index_t nb_elements_per_line=*,
                     list elements_names=*,
                     str avl_name=*,
                     str comments=*,
                     bint create=*
                    )
    cpdef select_line(self, index_t line_nb)
    cpdef select_lines(self, list line_selection)
    cpdef save_and_close(self)


cdef class OBIView_NUC_SEQS(OBIView):

    cdef OBIDMS_column ids
    cdef OBIDMS_column sequences
    cdef OBIDMS_column descriptions
    
    cpdef delete_column(self, str column_name)
    

cdef class OBIView_line :

    cdef index_t index
    cdef OBIView view


cdef class OBIDMS:

    cdef OBIDMS_p pointer
    cdef str dms_name
    
    cpdef close(self)
    cpdef OBI_Taxonomy open_taxonomy(self, str taxo_name)
    cpdef OBIView open_view(self, str view_name)
    cpdef OBIView new_view(self, str view_name, object view_to_clone=*, list line_selection=*, str view_type=*, str comments=*)
    cpdef dict read_view_infos(self, str view_name)
    cpdef dict read_views(self)

