#cython: language_level=3

from .obitypes           cimport const_char_p, \
                                 OBIType_t, \
                                 obiversion_t, \
                                 index_t, \
                                 time_t
from ..capi.obidms       cimport OBIDMS_p
from ..capi.obidmscolumn cimport OBIDMS_column_p


cdef extern from "obiview.h" nogil:

    extern const_char_p VIEW_TYPE_NUC_SEQS
    extern const_char_p NUC_SEQUENCE_COLUMN
    extern const_char_p ID_COLUMN
    extern const_char_p DESCRIPTION_COLUMN

    struct Column_reference_t :
        const_char_p  column_name
        obiversion_t  version
    
    ctypedef Column_reference_t* Column_reference_p


    struct Obiview_t :
        OBIDMS_p            dms
        const_char_p        name
        OBIDMS_column_p     line_selection
        OBIDMS_column_p     new_line_selection
        OBIDMS_column_p     columns
        bint                read_only
        Column_reference_t  line_selection_reference
        index_t             line_count
        int                 column_count
        const_char_p        comments

    ctypedef Obiview_t* Obiview_p


    struct Obiview_infos_t :
        int                 view_number
        int                 column_count
        index_t             line_count
        const_char_p        name
        const_char_p        created_from
        time_t              creation_date
        bint                all_lines
        Column_reference_t  line_selection
        Column_reference_p  column_references
        const_char_p        view_type
        const_char_p        comments

    ctypedef Obiview_infos_t* Obiview_infos_p


    struct Obiviews_header_t :
        size_t header_size
        size_t views_size
        int    view_count
    
    ctypedef Obiviews_header_t* Obiviews_header_p


    struct Obiviews_infos_all_t :
        Obiviews_header_p    header
        Obiview_infos_p      view_infos

    ctypedef Obiviews_infos_all_t* Obiviews_infos_all_p


    Obiview_p obi_new_view_nuc_seqs(OBIDMS_p dms, const_char_p view_name, Obiview_p view_to_clone, index_t* line_selection, const_char_p comments)

    Obiview_p obi_new_view(OBIDMS_p dms, const_char_p view_name, Obiview_p view_to_clone, index_t* line_selection, const_char_p comments)
    
    Obiview_p obi_new_view_cloned_from_name(OBIDMS_p dms, const_char_p view_name, const_char_p view_to_clone_name, index_t* line_selection, const_char_p comments)

    Obiview_p obi_new_view_nuc_seqs_cloned_from_name(OBIDMS_p dms, const_char_p view_name, const_char_p view_to_clone_name, index_t* line_selection, const_char_p comments)
              
    Obiview_p obi_open_view(OBIDMS_p dms, const_char_p view_name)
    
    Obiviews_infos_all_p obi_read_views(OBIDMS_p dms)

    int obi_unmap_read_views(Obiviews_infos_all_p views)

    int obi_view_add_column(Obiview_p view,
                            const_char_p column_name,
                            obiversion_t version_number,
                            OBIType_t data_type,
                            index_t nb_lines,
                            index_t nb_elements_per_line,
                            const_char_p elements_names,
                            const_char_p avl_name,
                            const_char_p comments,
                            bint create)

    int obi_view_delete_column(Obiview_p view, const_char_p column_name)
    
    int obi_select_line(Obiview_p view, index_t line_nb)

    int obi_select_lines(Obiview_p view, index_t* line_nbs)

    OBIDMS_column_p obi_view_clone_column(Obiview_p view, const_char_p column_name)
        
    OBIDMS_column_p obi_view_get_column(Obiview_p view, const_char_p column_name)

    OBIDMS_column_p* obi_view_get_pointer_on_column_in_view(Obiview_p view, const_char_p column_name)

    int obi_save_view(Obiview_p view)
    
    int obi_close_view(Obiview_p view)
    
    int obi_save_and_close_view(Obiview_p view)

